class MockDB:
    def __init__(self):
        self.session = None
        self.user = None

    async def get_session(self, **kwargs):
        return self.session

    async def set_session(self, session):
        self.session = session

    async def new_session(self, **kwargs):
        return kwargs

    async def get_user(self, **kwargs):
        return self.user

    async def set_user(self, user):
        self.user = user

    async def new_user(self, **kwargs):
        return kwargs



import logging
from motor import motor_asyncio

logger = logging.getLogger(__name__)


class MongodbInterface:
    """
    https://github.com/mongodb/motor
    """

    def __init__(self,
                 uri='localhost',
                 db_name='bots',
                 user_collection_name='user',
                 session_collection_name='session',
                 ):
        self.cx = None
        self.db = None
        self.session_collection = None
        self.user_collection = None
        self.uri = uri
        self.db_name = db_name
        self.session_collection_name = session_collection_name
        self.user_collection_name = user_collection_name

    async def connect(self, loop):
        self.cx = motor_asyncio.AsyncIOMotorClient(self.uri, io_loop=loop)
        self.db = self.cx.get_database(self.db_name)
        self.user_collection = self.db.get_collection(self.user_collection_name)
        self.session_collection = self.db.get_collection(self.session_collection_name)

    async def clear_collections(self):
        await self.session_collection.drop()
        self.session_collection = self.db.get_collection(self.session_collection_name)
        await self.user_collection.drop()
        self.user_collection = self.db.get_collection(self.user_collection_name)

    async def get_session(self, **kwargs):
        return await self.session_collection.find_one(kwargs)

    async def set_session(self, session):
        old_session = await self.session_collection.find_one({'user_id': session['user_id']})
        logger.debug('old_session')
        logger.debug(old_session)
        if not old_session:
            res = await self.session_collection.insert(session)
        else:
            res = await self.session_collection.update({'user_id': session['user_id']}, session)

        return res

    async def new_session(self, user, **kwargs):
        kwargs['user_id'] = kwargs.get('user_id', user['_id'])
        kwargs['stack'] = kwargs.get('stack', [])
        id = await self.session_collection.insert(kwargs)
        return await self.session_collection.find_one({'_id': id})

    async def get_user(self, **kwargs):
        if 'id' in kwargs:
            kwargs['_id'] = kwargs.get('id', None)
            del kwargs['id']
        return await self.user_collection.find_one(kwargs)

    async def set_user(self, user):
        if not getattr(user, '_id', None):
            res = await self.user_collection.insert(user)
        else:
            res = await self.user_collection.update({'_id': user._id}, user)
        return res

    async def new_user(self, **kwargs):
        id = await self.user_collection.insert(kwargs)
        return await self.user_collection.find_one({'_id': id})



import asyncio
import logging

from .middlewares import any, location, text

logger = logging.getLogger(__name__)
interfaces = {}


async def ask(body, options=None, user=None):
    """
    simple ask with predefined options

    :param body:
    :param options: (optional)  in form of
    {'title': <message>, 'payload': <any json>}
    :param user:
    :return:
    """
    await send_text_message_to_all_interfaces(
        recipient=user, text=body, options=options)
    return any.Any()


# TODO: move to middlewares/location/location.py and make async

def ask_location(body, user):
    # TODO:
    # 1 ask user
    say(body, user)

    # 2 wait for answer
    # 3 process answer
    # 4 ask details once we not sure
    return [location.Any(), text.Any()]


async def say(body, user):
    """
    say something to user

    :param body:
    :param user:
    :return:
    """
    return await send_text_message_to_all_interfaces(
        recipient=user, text=body)


async def send_text_message_to_all_interfaces(*args, **kwargs):
    """
    TODO:
    we should know from where user has come and use right interface
    as well right interface can be chosen

    :param args:
    :param kwargs:
    :return:
    """
    logger.debug('async_send_text_message_to_all_interfaces')
    tasks = [interface.send_text_message(*args, **kwargs)
             for type, interface in interfaces.items()]

    logger.debug('  tasks')
    logger.debug(tasks)

    res = [body for body in await asyncio.gather(*tasks)]
    logger.debug('  res')
    logger.debug(res)
    return res


def add_http(http):
    logger.debug('add_http')
    logger.debug(http)
    for _, interface in interfaces.items():
        interface.add_http(http)


def add_interface(interface):
    logger.debug('add_interface')
    logger.debug(interface)
    interfaces[interface.type] = interface
    return interface



from ... import matchers


@matchers.matcher()
class Any:
    type = 'Option.Any'

    def __init__(self):
        pass

    def validate(self, message):
        return message.get('data', {}).get('option', False)


@matchers.matcher()
class Match:
    type = 'Option.Match'

    def __init__(self, option):
        self.option = option

    def validate(self, message):
        return message.get('data', {}).get('option', None) == self.option

    def serialize(self):
        return self.option

    @staticmethod
    def deserialize(option):
        return Match(option)



import asyncio
import logging

from . import chat
from .ast import callable as callable_module, common, \
    forking, library, parser, processor

logger = logging.getLogger(__name__)

# instantiate handlers

stories_library = library.StoriesLibrary()

parser_instance = parser.Parser()

story_processor_instance = processor.StoryProcessor(
    parser_instance,
    stories_library,
    middlewares=[forking.Middleware()]
)

common_stories_instance = common.CommonStoriesAPI(
    parser_instance,
    stories_library)

callable_stories_instance = callable_module.CallableStoriesAPI(
    library=stories_library,
    parser_instance=parser_instance,
    processor_instance=story_processor_instance,
)

forking_api = forking.ForkingStoriesAPI(
    parser_instance=parser_instance,
)

# expose story API:

callable = callable_stories_instance.callable
case = forking_api.case
on = common_stories_instance.on
part = common_stories_instance.part

# expose message handler API:

match_message = story_processor_instance.match_message

# expose operators

EndOfStory = callable_module.EndOfStory
Switch = forking.Switch
SwitchOnValue = forking.SwitchOnValue


def check_spec(spec, obj):
    for method in [getattr(obj, s, None) for s in spec]:
        if not method:
            return False
    return True


middlewares = []


def use(middleware):
    """
    attache middleware

    :param middleware:
    :return:
    """

    logger.debug('use')
    logger.debug(middleware)

    middlewares.append(middleware)

    # TODO: maybe it is good time to start using DI (dependency injection)

    if check_spec(['send_text_message'], middleware):
        chat.add_interface(middleware)

    if check_spec(['handle'], middleware):
        story_processor_instance.add_interface(middleware)

    if check_spec(['get_user', 'set_user', 'get_session', 'set_session'], middleware):
        story_processor_instance.add_storage(middleware)

    if check_spec(['post', 'webhook'], middleware):
        chat.add_http(middleware)

    return middleware


async def start():
    await asyncio.gather(
        *[m.start() for m in middlewares if hasattr(m, 'start')]
    )


async def stop():
    await asyncio.gather(
        *[m.stop() for m in middlewares if hasattr(m, 'stop')]
    )


def forever(loop):
    try:
        loop.run_forever()
    except KeyboardInterrupt:  # pragma: no cover
        pass
    finally:
        loop.run_until_complete(stop())



import asyncio
import logging
import inspect

from .. import matchers
from . import parser, callable, forking

logger = logging.getLogger(__name__)


class StoryProcessor:
    def __init__(self, parser_instance, library, middlewares=[]):
        self.interfaces = []
        self.library = library
        self.middlewares = middlewares
        self.parser_instance = parser_instance
        self.storage = None

    def add_interface(self, interface):
        if self.storage:
            interface.add_storage(self.storage)
        self.interfaces.append(interface)
        interface.processor = self

    def add_storage(self, storage):
        self.storage = storage
        for interface in self.interfaces:
            interface.add_storage(storage)

    async def match_message(self, message):
        logger.debug('')
        logger.debug('> match_message <')
        logger.debug('')
        logger.debug('  {} '.format(message))
        session = message['session']
        if len(session['stack']) > 0:
            logger.debug('  check stack')
            logger.debug('    session.stack = {}'.format(session['stack']))
            stack_tail = None
            while True:
                if len(session['stack']) == 0:
                    # we have reach the bottom of stack
                    # if stack_tail and stack_tail['data']:
                    #     validator = matchers.deserialize(stack_tail['data'])
                    #     if validator.validate(message):
                    #         # it seems that last callee in a stack should receive
                    #         # this message but it doesn't have more story parts
                    #         # so we just skip this message
                    #         return
                    # stack_tail = None
                    logger.debug('  we have reach the bottom of stack '
                                 'so no once has receive this message')
                    return None
                stack_tail = session['stack'].pop()
                if not stack_tail['data']:
                    break
                validator = matchers.deserialize(stack_tail['data'])
                if hasattr(validator, 'immediately') and validator.immediately:
                    return
                if stack_tail['step'] < len(
                        self.library.get_story_by_topic(stack_tail['topic'], stack=session['stack']).story_line
                ):
                    # if we haven't reach last step in list of story so we can parse result
                    break

            logger.debug('    after check session.stack = {}'.format(session['stack']))
            if stack_tail:
                logger.debug('      topic {}'.format(stack_tail['topic']))
                logger.debug('      step {}'.format(stack_tail['step']))
            logger.debug('      stack_tail = {}'.format(stack_tail))
            if stack_tail and stack_tail['data']:
                logger.debug('  got it!')
                validator = matchers.deserialize(stack_tail['data'])
                validation_result = validator.validate(message)
                logger.debug('      validation_result {}'.format(validation_result))
                if not not validation_result:
                    return await self.process_next_part_of_story({
                        'step': stack_tail['step'],
                        'story': self.library.get_story_by_topic(stack_tail['topic'], stack=session['stack']),
                        'stack_tail': [stack_tail],
                    },
                        validation_result, session, message,
                        bubble_up=True)

        if len(session['stack']) == 0:
            session['stack'] = [build_empty_stack_item()]

        compiled_story = self.library.get_right_story(message)
        if not compiled_story:
            return
        return await self.process_story(
            idx=0,
            message=message,
            compiled_story=compiled_story,
            session=session,
            bubble_up=True,
        )

    async def process_story(self, session, message, compiled_story, idx=0, story_args=[], story_kwargs={},
                            bubble_up=True):
        logger.debug('')
        logger.debug('process_story')
        logger.debug('')

        logger.debug('  bubble_up {}'.format(bubble_up))
        logger.debug('! topic {}'.format(compiled_story.topic))
        logger.debug('! step {}'.format(idx))
        logger.debug('  story {}'.format(compiled_story))
        logger.debug('  message {}'.format(message))
        logger.debug('  session.stack {} ({})'.format(session['stack'], len(session['stack'])))
        logger.debug(
            '  previous_topics: {}'.format(session['stack'][-2]['topic'] if len(session['stack']) > 1 else None))

        story_line = compiled_story.story_line

        current_stack_level = len(session['stack']) - 1
        session['stack'][-1]['topic'] = compiled_story.topic

        waiting_for = None

        while idx < len(story_line):
            logger.debug('')
            logger.debug('  next iteration of {}'.format(compiled_story.topic))
            logger.debug('      idx = {} ({})'.format(idx, len(story_line)))
            logger.debug('      session {}'.format(session['stack']))

            story_part = story_line[idx]

            logger.debug('  going to call: {}'.format(story_part.__name__))

            # TODO: just should skip story part
            # but it should be done in process_next_part_of_story
            if not isinstance(story_part, parser.StoryPartFork):
                if message:
                    # process common story part
                    waiting_for = story_part(message)
                else:
                    # process startpoint of callable story
                    waiting_for = story_part(*story_args, **story_kwargs)

                if inspect.iscoroutinefunction(story_part):
                    waiting_for = await waiting_for

                logger.debug('  got result {}'.format(waiting_for))

            idx += 1
            if len(session['stack']) > current_stack_level:
                session['stack'][current_stack_level]['step'] = idx

            # TODO: should be refactor and put somewhere
            if waiting_for:
                # story part return value so we should somehow react on it
                if isinstance(waiting_for, forking.SwitchOnValue):
                    # SwitchOnValue is so special because it is the only result
                    # that doesn't async.
                    waiting_for = await forking.process_switch_on_value(compiled_story,
                                                                        idx, message, self, session, waiting_for)

                    if waiting_for:
                        logger.debug('  bubble up')
                        if bubble_up and isinstance(waiting_for, callable.EndOfStory):
                            break
                        else:
                            # if processed story part is waiting for result
                            # neither this story should continue working
                            return waiting_for

                elif isinstance(waiting_for, callable.EndOfStory):
                    if bubble_up:
                        break
                    return callable.process_end_of_story(message, waiting_for)
                else:
                    return process_async_operation(compiled_story, current_stack_level,
                                                   idx, self, session, waiting_for)

        # current story line is over
        if len(session['stack']) > 1:
            logger.debug('  we can bobble by stack')
            # but it seems that we have hierarchy of callable
            # stories so we should drop current stack element
            # because it is over and return to the previous story

            logger.debug('  action: reduce stack -1')
            session['stack'].pop()
            logger.debug('  session.stack = {}'.format(session['stack']))

            if message:
                if bubble_up:
                    waiting_for = await self.match_message(message)
                else:
                    logger.debug('  we reject bubbling in this call')

        return waiting_for

    async def process_next_part_of_story(self, received_data, validation_result, session, message, bubble_up=True):
        logger.debug('')
        logger.debug('process_next_part_of_story')
        logger.debug('')
        logger.debug('  topic {}'.format(received_data['story'].topic))
        logger.debug('  step {} ({})'.format(received_data['step'], len(received_data['story'].story_line)))

        for m in self.middlewares:
            if hasattr(m, 'process'):
                received_data = m.process(received_data, validation_result)

        logger.debug('  len(session.stack) = {}'.format(len(session['stack'])))
        logger.debug('  session.stack = {}'.format(session['stack']))
        logger.debug('  action: extend stack by +{}'.format(len(received_data['stack_tail'])))

        session['stack'].extend(received_data['stack_tail'])
        session['stack'][-1] = build_empty_stack_item()

        logger.debug('  session.stack = {}'.format(session['stack']))
        logger.debug('! after topic {}'.format(received_data['story'].topic))
        logger.debug('! after step {}'.format(received_data['step']))

        # we shouldn't bubble up because we inside other story
        # that under control
        return await self.process_story(
            idx=received_data['step'],
            message=message,
            compiled_story=received_data['story'],
            session=session,
            bubble_up=bubble_up,
        )


def process_async_operation(compiled_story, current_stack_level, idx, processor, session, waiting_for):
    # should wait result of async operation
    # (for example answer from user)
    for m in processor.middlewares:
        if hasattr(m, 'process_validator'):
            waiting_for = m.process_validator(processor, waiting_for, compiled_story)

    validator = matchers.get_validator(waiting_for)

    logger.debug('  before serialize validator')

    session['stack'][current_stack_level] = {
        'type': validator.type,
        'data': matchers.serialize(validator),
        'step': idx,
        'topic': compiled_story.topic,
    }
    return waiting_for


def build_empty_stack_item():
    return {
        'data': None,
        'step': 0,
        'topic': None,
    }



from aiohttp import test_utils
import pytest
from . import AioHttpInterface
from ..tests.fake_server import fake_fb


@pytest.mark.asyncio
async def test_post(event_loop):
    async with fake_fb.Server(event_loop) as server:
        async with server.session() as session:
            http = AioHttpInterface()
            http.session = session

            assert await http.post(fake_fb.URI.format('/v2.6/me/messages/'), json={'message': 'hello world!'})
            assert len(server.history) == 1
            req = server.history[-1]['request']
            assert req.content_type == 'application/json'
            assert await req.json() == {'message': 'hello world!'}


@pytest.mark.asyncio
async def test_listen_webhook(event_loop):
    handler = test_utils.make_mocked_coro(return_value={
        'result': 'ok',
    })
    http = AioHttpInterface(loop=event_loop, port=9876)
    http.webhook(uri='/webhook', handler=handler)
    try:
        await http.start()
        res = await http.post('http://localhost:9876/webhook', json={'message': 'Is there anybody in there?'})
        handler.assert_called_once_with({'message': 'Is there anybody in there?'})
        assert res == {'result': 'ok'}
    finally:
        await http.stop()


@pytest.mark.asyncio
async def test_should_not_create_server_if_there_wasnt_any_webhooks(event_loop):
    http = AioHttpInterface(loop=event_loop, port=9876)
    try:
        await http.start()
        assert not http.server
    finally:
        await http.stop()



import aiohttp
import asyncio
import logging
import json as _json
from aiohttp import web
from yarl import URL

logger = logging.getLogger(__name__)


class WebhookHandler:
    def __init__(self, handler):
        self.handler = handler

    async def handle(self, request):
        res = await self.handler(await request.json())
        return web.Response(text=_json.dumps(res))


class AioHttpInterface:
    type = 'interface.aiohttp'

    def __init__(self, loop=None, host='0.0.0.0', port=None,
                 shutdown_timeout=60.0, ssl_context=None,
                 backlog=128,
                 ):
        if port is None:
            if not ssl_context:
                port = 8080
            else:
                port = 8443

        self.backlog = backlog
        self.loop = loop or asyncio.get_event_loop()
        self.host = host
        self.port = port
        self.shutdown_timeout = shutdown_timeout

        self.ssl_context = ssl_context

        self.app = None
        self.session = None
        self.server = None
        self.handler = None

    async def post(self, url, params=None, headers=None, json=None):
        logger.debug('post url={}'.format(url))
        headers = headers or {}
        headers['Content-Type'] = headers.get('Content-Type', 'application/json')
        with aiohttp.ClientSession(loop=self.loop) as session:
            # be able to mock session from outside
            session = self.session or session
            resp = await session.post(
                url,
                params=params,
                headers=headers,
                data=_json.dumps(json),
            )
            return await resp.json()

    def get_app(self):
        if not self.has_app():
            logger.debug('create web app')
            self.app = web.Application(
                loop=self.loop,
            )
        return self.app

    def has_app(self):
        return self.app is not None

    def webhook(self, uri, handler):
        logger.debug('webhook {}'.format(uri))
        self.get_app().router.add_post(uri, WebhookHandler(handler).handle)

    async def start(self):
        if not self.has_app():
            return
        logger.debug('start')
        app = self.get_app()
        handler = app.make_handler()
        server = self.loop.create_server(
            handler,
            self.host,
            self.port,
            ssl=self.ssl_context,
            backlog=self.backlog,
        )

        srv, startup_res = await asyncio.gather(
            server, app.startup(),
            loop=self.loop)

        scheme = 'https' if self.ssl_context else 'http'
        url = URL('{}://localhost'.format(scheme))
        url = url.with_host(self.host).with_port(self.port)
        logger.debug('======== Running on {} ========\n'
                     '(Press CTRL+C to quit)'.format(url))

        self.server = srv
        self.handler = handler

    async def stop(self):
        if not self.has_app():
            return
        logger.debug('stop')
        self.server.close()
        await self.server.wait_closed()
        await self.app.shutdown()
        await self.handler.finish_connections(self.shutdown_timeout)
        await self.app.cleanup()



import logging

logger = logging.getLogger(__name__)


class FBInterface:
    type = 'interface.facebook'

    def __init__(self,
                 api_uri='https://graph.facebook.com/v2.6',
                 token=None,
                 webhook=None,
                 ):
        """

        :param token: should take from os.environ['FB_ACCESS_TOKEN']
        """
        self.api_uri = api_uri
        self.token = token
        self.webhook = webhook

        self.http = None
        self.processor = None
        self.storage = None

    async def send_text_message(self, recipient, text, options=None):
        """
        async send message to the facebook user (recipient)

        :param session:
        :param recipient:
        :param text:
        :param options:

        :return:
        """

        if not options:
            options = []

        message = {
            'text': text,
        }

        quick_replies = [{**reply, 'content_type': 'text'} for reply in options]
        if len(quick_replies) > 0:
            message['quick_replies'] = quick_replies

        return await self.http.post(
            self.api_uri + '/me/messages/',
            params={
                'access_token': self.token,
            },
            json={
                'recipient': {
                    'id': recipient['facebook_user_id'],
                },
                'message': message,
            })

    def add_http(self, http):
        logger.debug('add_http')
        logger.debug(http)
        self.http = http
        if self.webhook:
            http.webhook(self.webhook, self.handle)

    def add_storage(self, storage):
        logger.debug('add_storage')
        logger.debug(storage)
        self.storage = storage

    async def handle(self, data):
        logger.debug('')
        logger.debug('> handle <')
        logger.debug('')
        logger.debug('  entry: {}'.format(data))
        for e in data.get('entry', []):
            messaging = e.get('messaging', [])
            logger.debug('  messaging: {}'.format(messaging))

            if len(messaging) == 0:
                logger.warning('  entry {} list lack of "messaging" field'.format(e))

            for m in messaging:
                logger.debug('  m: {}'.format(m))

                facebook_user_id = m['sender']['id']

                user = await self.storage.get_user(facebook_user_id=facebook_user_id)
                if not user:
                    logger.debug('  should create new user {}'.format(facebook_user_id))
                    user = await self.storage.new_user(
                        facebook_user_id=facebook_user_id,
                    )

                    """
                    Make request to facebook
                    to receive more information about user

                    curl -X GET "https://graph.facebook.com/v2.6/<USER_ID>?fields=first_name,last_name,profile_pic,locale,timezone,gender&access_token=PAGE_ACCESS_TOKEN"


                    "first_name": "Peter",
                    "last_name": Chang",
                    "profile_pic": "https://fbcdn-profile-a.akamaihd.net/hprofile-ak-xpf1/v/t1.0-1/p200x200/13055603_10105219398495383_8237637584159975445_n.jpg?oh=1d241d4b6d4dac50eaf9bb73288ea192&oe=57AF5C03&__gda__=1470213755_ab17c8c8e3a0a447fed3f272fa2179ce",
                    "locale": "en_US",
                    "timezone": -7,
                    "gender": "male"


                    More: https://developers.facebook.com/docs/messenger-platform/user-profile

                    """

                session = await self.storage.get_session(facebook_user_id=facebook_user_id)
                if not session:
                    logger.debug('  should create new session for user {}'.format(facebook_user_id))
                    session = await self.storage.new_session(
                        facebook_user_id=facebook_user_id,
                        user=user,
                    )

                message = {
                    'session': session,
                    'user': user,
                }
                raw_message = m.get('message', {})

                if raw_message == {}:
                    logger.warning('  entry {} "message" field is empty'.format(e))

                logger.debug('  raw_message: {}'.format(raw_message))

                data = {}
                text = raw_message.get('text', None)
                if text is not None:
                    data['text'] = {
                        'raw': text,
                    }
                else:
                    logger.warning('  entry {} "text"'.format(e))

                quick_reply = raw_message.get('quick_reply', None)
                if quick_reply is not None:
                    data['option'] = quick_reply['payload']

                message['data'] = data

                await self.processor.match_message(message)



from . import any
from ... import matchers


def test_serialize_any():
    m_old = any.Any()
    m_new = matchers.deserialize(matchers.serialize(m_old))
    assert isinstance(m_new, any.Any)


def test_serialize_any_of():
    m_old = any.AnyOf([any.Any(), any.Any()])
    m_new = matchers.deserialize(matchers.serialize(m_old))
    assert isinstance(m_new, any.AnyOf)
    assert len(m_new.list_of_matchers) == 2
    assert isinstance(m_new.list_of_matchers[0], any.Any)
    assert isinstance(m_new.list_of_matchers[1], any.Any)



import logging
import itertools

from . import parser

logger = logging.getLogger(__name__)


class StoriesLibrary:
    """
    storage of all available stories
    """

    def __init__(self):
        self.message_handling_stories = []
        self.callable_stories = []

    def clear(self):
        self.message_handling_stories = []
        self.callable_stories = []

    def add_message_handler(self, story):
        self.message_handling_stories.append(story)

    def add_callable(self, story):
        self.callable_stories.append(story)

    def get_callable_by_topic(self, topic):
        return [s for s in self.callable_stories if s.topic == topic][0]

    def get_right_story(self, message):
        matched_stories = [story for story in self.message_handling_stories
                           if story.extensions['validator'].validate(message)]
        return matched_stories[0] if len(matched_stories) > 0 else None

    def get_story_by_topic(self, topic, stack=[]):
        """
        get story and take about context stack
        :param topic:
        :param stack:
        :return:
        """
        options = [s for s in [*self.callable_stories, *self.message_handling_stories] if s.topic == topic]

        if len(options) > 0:
            return options[0]

        if len(options) == 0 and stack and len(stack) > 0:
            parent = self.get_story_by_topic(stack[-1]['topic'], stack[:-2])
            if not parent:
                return None
            inner_stories = [
                story.children for story in parent.story_line if isinstance(story, parser.StoryPartFork)
            ]

            inner_stories = [item for sublist in inner_stories for item in sublist]

            child_options = [story for story in inner_stories if story.topic == topic]
            if len(child_options) == 0:
                return None
            elif len(child_options) == 1:
                return child_options[0]
            else:
                raise Error('We have few options with the same name {}'.format(topic))

        return None
